<?php
/**
 * @package   DPMedia
 * @copyright Copyright (C) 2021 Digital Peak GmbH. <https://www.digital-peak.com>
 * @license   https://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 */

namespace DigitalPeak\Plugin\Filesystem\DPFtp\Adapter;

use DigitalPeak\Library\DPMedia\Adapter\Adapter;
use DigitalPeak\Library\DPMedia\Adapter\DownloadMediaTrait;
use DigitalPeak\Library\DPMedia\Adapter\MimeTypeMapping;
use DigitalPeak\Plugin\Filesystem\DPFtp\Client\ClientAwareInterface;
use DigitalPeak\Plugin\Filesystem\DPFtp\Client\ClientAwareTrait;
use DigitalPeak\ThinHTTP\ClientInterface;
use Joomla\CMS\Application\CMSApplication;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Uri\Uri;
use Joomla\Component\Media\Administrator\Exception\FileNotFoundException;
use Joomla\Database\DatabaseInterface;
use Joomla\Filesystem\Path;
use Joomla\Registry\Registry;

/**
 * Read only FTP adapter for Joomla media manager.
 */
class FtpAdapter extends Adapter implements ClientAwareInterface
{
	use ClientAwareTrait;
	use DownloadMediaTrait;

	protected bool $useLastPathSegment = false;

	public function __construct(
		Registry $config,
		ClientInterface $http,
		MimeTypeMapping $mimeTypeMapping,
		DatabaseInterface $db,
		CMSApplication $app
	) {
		$config->set('local_media', 1);
		parent::__construct($config, $http, $mimeTypeMapping, $db, $app);
	}

	protected function fetchFile(string $path = '/'): \stdClass
	{
		// Somehow on create operations $path has //
		$path = str_replace('//', '/', Path::clean($path, '/'));
		$file = array_filter($this->fetchFiles(dirname($path)), static fn ($f): bool => $f->path === $path);
		if ($file === []) {
			throw new FileNotFoundException('File not found');
		}

		return reset($file);
	}

	protected function fetchFiles(string $path = '/'): array
	{
		$path = $this->getPath($path) !== '' && $this->getPath($path) !== '0' ? $this->getPath($path) : '/';

		$ftpPath = $path;
		if (str_contains($path, '.')) {
			$ftpPath = dirname($path);
		}

		$this->getClient()->connect($this->getConfig());
		$files = $this->getClient()->getFiles($ftpPath);

		$data = [];
		foreach ($files as $entry) {
			if ($entry['name'] === '.') {
				continue;
			}
			if ($entry['name'] === '..') {
				continue;
			}
			// When the path is different, then a file is fetched and the list should only contain the file
			if ($ftpPath !== $path && $ftpPath . '/' . $entry['name'] !== $path) {
				continue;
			}

			$data[] = $this->getFileInfo((object)$entry, $ftpPath);
		}

		return $data;
	}

	protected function fetchUrl(string $path): string
	{
		return rtrim(Uri::root(), '/') . $this->download($this->getFile($path), $this->getConfig());
	}

	protected function fetchSearch(string $path, string $needle, bool $recursive = false): array
	{
		$files = $this->getFiles($path);

		$data = [];
		foreach ($files as $file) {
			if (!str_contains((string)$file->name, $needle)) {
				continue;
			}
			$data[] = $file;
		}

		return $data;
	}

	/**
	 * Extract file information from an entry of FTP.
	 *
	 *
	 * @return \stdClass
	 */
	private function getFileInfo(\stdClass $fileEntry, string $path): \stdClass
	{
		$file       = new \stdClass();
		$file->type = $fileEntry->type === 'file' ? 'file' : 'dir';
		$file->name = $fileEntry->name;
		$file->path = rtrim($path, '/') . '/' . $fileEntry->name;
		$file->path = substr_replace($file->path, '', 0, strlen(rtrim((string)$this->getConfig()->get('root_folder', '/'), '/')));

		$file->size      = empty($fileEntry->size) ? (empty($fileEntry->sizd) ? 0 : (int)$fileEntry->sizd) : ((int)$fileEntry->size);
		$file->width     = 0;
		$file->height    = 0;
		$file->extension = '';
		$file->mime_type = '';

		if ($file->type === 'file') {
			$file->extension = pathinfo((string)$file->name, PATHINFO_EXTENSION);
			$file->mime_type = $this->mimeTypeMapping->getMimetype($file->extension);
		}

		$date       = empty($fileEntry->modify) ? \DateTime::createFromFormat('YmdHis', $fileEntry->modify) : null;
		$createDate = $this->getDate($date ? $date->format('c') : null);
		$updateDate = clone $createDate;

		$file->create_date_formatted   = HTMLHelper::_('date', $createDate, $this->app->getLanguage()->_('DATE_FORMAT_LC5'));
		$file->create_date             = $createDate->format('c');
		$file->modified_date_formatted = HTMLHelper::_('date', $updateDate, $this->app->getLanguage()->_('DATE_FORMAT_LC5'));
		$file->modified_date           = $updateDate->format('c');

		$file->thumb_path = $this->generateThumb($file, $this->getConfig());

		return $file;
	}

	protected function getContent(\stdclass $file, Registry $config): string
	{
		$this->getClient()->connect($this->getConfig());
		return $this->getClient()->download($this->getPath($file->path));
	}
}
